//
//  fs.swift
//  Noze.IO
//
//  Created by Helge Hess on 02/07/15.
//  Copyright © 2015 ZeeZide GmbH. All rights reserved.
//

import Dispatch
import core
import streams

public class NozeFS : NozeModule {
  
  // A queue which is used by all FS functions to do async operations (not
  // covered by GCD)
  lazy var Q = DispatchQueue.global()
}
public let module = NozeFS()


// MARK: - Common type aliases

public typealias DataCB   = ( Error?, [ UInt8 ]? ) -> Void
public typealias StringCB = ( Error?, String?    ) -> Void
public typealias ErrorCB  = ( Error?             ) -> Void


#if !os(Linux) // 2016-09-12: Not yet available on Linux
// MARK: - Watch Files or Directories. Get notified on changes.

@discardableResult
public func watch(_ filename : String,
                  persistent : Bool = true,
                  recursive  : Bool = false,
                  listener   : FSWatcherCB? = nil) -> FSWatcher
{
  let dirWatch : Bool = {
    guard let finfo = try? fs.statSync(filename) else { return false }
    return finfo.isDirectory()
  }()
  
  if dirWatch {
    return FSDirWatcher(filename, persistent: persistent,
                        recursive: recursive,
                        listener: listener)
  }
  
  // TBD: throw an error if recursive is true?
  return FSRawWatcher(filename, persistent: persistent,
                      listener: listener)
}
#endif /* !Linux */

